<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class Cooked_Pro_Session {

	private $session;
	private $use_php_sessions = false;
	private $prefix = '';

	public function __construct() {

		$this->use_php_sessions = $this->use_php_sessions();

		if( $this->use_php_sessions ) {

			if( is_multisite() ) {

				$this->prefix = '_' . get_current_blog_id();

			}

			// Use PHP SESSION (must be enabled via the COOKED_USE_PHP_SESSIONS constant)
			add_action( 'init', array( $this, 'maybe_start_session' ), -2 );

		} else {

			if( ! $this->should_start_session() ) {
				return;
			}

			if ( ! defined( 'WP_SESSION_COOKIE' ) ) {
				define( 'WP_SESSION_COOKIE', 'cooked_wp_session' );
			}

			add_filter( 'wp_session_expiration_variant', array( $this, 'set_expiration_variant_time' ), 99999 );
			add_filter( 'wp_session_expiration', array( $this, 'set_expiration_time' ), 99999 );

		}

		if ( empty( $this->session ) && ! $this->use_php_sessions ) {
			add_action( 'plugins_loaded', array( $this, 'init' ), -1 );
		} else {
			add_action( 'init', array( $this, 'init' ), -1 );
		}

	}

	public function init() {

		if( $this->use_php_sessions ) {
			$this->session = isset( $_SESSION['cooked' . $this->prefix ] ) && is_array( $_SESSION['cooked' . $this->prefix ] ) ? $_SESSION['cooked' . $this->prefix ] : array();
		} else {
			$this->session = WP_Session::get_instance();
		}

		return $this->session;
	}

	public function get_id() {
		return $this->session->session_id;
	}

	public function get( $key ) {
		$key = sanitize_key( $key );
		return isset( $this->session[ $key ] ) ? maybe_unserialize( $this->session[ $key ] ) : false;
	}

	public function set( $key, $value ) {

		$key = sanitize_key( $key );

		if ( is_array( $value ) ) {
			$this->session[ $key ] = serialize( $value );
		} else {
			$this->session[ $key ] = $value;
		}

		if( $this->use_php_sessions ) {

			$_SESSION['cooked' . $this->prefix ] = $this->session;
		}

		return $this->session[ $key ];
	}

	public function set_expiration_variant_time( $exp ) {
		return ( 30 * 60 * 23 );
	}

	public function set_expiration_time( $exp ) {
		return ( 30 * 60 * 24 );
	}

	public function use_php_sessions() {

		$ret = false;

		// If the database variable is already set, no need to run autodetection
		$cooked_use_php_sessions = (bool) get_option( 'cooked_use_php_sessions' );

		if ( ! $cooked_use_php_sessions ) {

			// Attempt to detect if the server supports PHP sessions
			if( function_exists( 'session_start' ) ) {

				$this->set( 'cooked_use_php_sessions', 1 );

				if( $this->get( 'cooked_use_php_sessions' ) ) {

					$ret = true;

					// Set the database option
					update_option( 'cooked_use_php_sessions', true );

				}

			}

		} else {
			$ret = $cooked_use_php_sessions;
		}

		// Enable or disable PHP Sessions based on the COOKED_USE_PHP_SESSIONS constant
		if ( defined( 'COOKED_USE_PHP_SESSIONS' ) && COOKED_USE_PHP_SESSIONS ) {
			$ret = true;
		} else if ( defined( 'COOKED_USE_PHP_SESSIONS' ) && ! COOKED_USE_PHP_SESSIONS ) {
			$ret = false;
		}

		return (bool) apply_filters( 'cooked_use_php_sessions', $ret );
	}

	public function should_start_session() {

		$start_session = true;

		if( ! empty( $_SERVER[ 'REQUEST_URI' ] ) ) {

			$blacklist = $this->get_blacklist();
			$uri       = ltrim( $_SERVER[ 'REQUEST_URI' ], '/' );
			$uri       = untrailingslashit( $uri );

			if( in_array( $uri, $blacklist ) ) {
				$start_session = false;
			}

			if( false !== strpos( $uri, 'feed=' ) ) {
				$start_session = false;
			}

		}

		return apply_filters( 'cooked_start_session', $start_session );

	}

	public function get_blacklist() {

		$blacklist = apply_filters( 'cooked_session_start_uri_blacklist', array(
			'feed',
			'feed/rss',
			'feed/rss2',
			'feed/rdf',
			'feed/atom',
			'comments/feed'
		) );

		// Look to see if WordPress is in a sub folder or this is a network site that uses sub folders
		$folder = str_replace( network_home_url(), '', get_site_url() );

		if( ! empty( $folder ) ) {
			foreach( $blacklist as $path ) {
				$blacklist[] = $folder . '/' . $path;
			}
		}

		return $blacklist;
	}

	/**
	 * Starts a new session if one hasn't started yet.
	 */
	public function maybe_start_session() {

		if( ! $this->should_start_session() ) {
			return;
		}

		if( ! session_id() && ! headers_sent() ) {
			session_start();
		}
	}

}